package mi.hdm.mealPlan;

import mi.hdm.recipes.NutritionCalculator;
import mi.hdm.recipes.NutritionTable;
import mi.hdm.recipes.Recipe;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.time.LocalDate;
import java.util.Map;
import java.util.Optional;

/**
 * Allow the user to create a meal plan for the week. The user can check out his nutrition scores for the week.
 */
public class MealPlan {
    private static final Logger log = LogManager.getLogger(MealPlan.class);
    private final Map<LocalDate, Recipe> plan;

    public MealPlan(Map<LocalDate, Recipe> plan) {
        this.plan = plan;
    }

    public void addRecipeToMealPlan(Recipe recipe, LocalDate date) {
        if (date == null) {
            throw new NullPointerException("Date can not be null");
        }

        LocalDate invalid = LocalDate.now().plusDays(7);

        if (date.isBefore(LocalDate.now()) || invalid.isBefore(date)) {
            throw new UnsupportedOperationException("You can only add recipes between today and in 7 days from now!");
        }
        plan.put(date, recipe);
    }

    public void clear(LocalDate date) {
        if (date == null) {
            throw new NullPointerException("Date can not be null");
        }
        if (plan.containsKey(date)) {
            log.info("Day {} cleared successfully.", date);
            plan.remove(date);
        } else {
            throw new NullPointerException("This date does not exist in the meal plan.");
        }
    }

    public void clear() {
        log.info("MealPlan cleared successfully.");
        plan.clear();
    }

    public Optional<Recipe> getRecipeForDay(LocalDate date) {
        return Optional.ofNullable(plan.get(date));
    }

    public Map<LocalDate, Recipe> getAllRecipesFromPlan() {
        return plan;
    }

    public void removePastRecipes() {
        final LocalDate now = LocalDate.now();
        for (LocalDate d : plan.keySet()) {
            if (d.isBefore(now)) plan.remove(d);
        }
    }

    public NutritionTable getNutritionTable() {
        return NutritionCalculator.calculateNutritionTable(this);
    }

}
