package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidIngredientException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;


public class IngredientTest {
    IngredientManager underTest = IngredientManager.getInstance();
    NutritionTable tt = ValidObjectsPool.getValidNutritionTableOne();

    @BeforeEach
    public void setup() {
        underTest.clearIngredients();
    }

    @Test
    public void shouldNotCreateIngredientWithNameNull() {
        Assertions.assertThrows(InvalidIngredientException.class, () -> new Ingredient(Measurement.GRAM, null, tt));
    }

    @Test
    public void shouldNotCreateIngredientWithNameEmpty() {
        Assertions.assertThrows(InvalidIngredientException.class, () -> new Ingredient(Measurement.GRAM, "", tt));
    }

    @Test
    public void shouldNotCreateIngredientWithNutritionTableNull() {
        Assertions.assertThrows(InvalidIngredientException.class, () -> new Ingredient(Measurement.GRAM, "foo", null));
    }

    @Test
    public void testEqualIngredients() {
        Ingredient i1 = new Ingredient(Measurement.GRAM, "Banana", tt);
        Ingredient i2 = new Ingredient(Measurement.PIECE, "Banana", tt);
        Ingredient i3 = new Ingredient(Measurement.GRAM, "Apple", tt);

        Assertions.assertEquals(i1, i2);
        Assertions.assertNotEquals(i1, i3);
        Assertions.assertNotEquals(i2, i3);
    }

    @Test
    public void shouldNotBeAbleToAddIdenticalName() {
        underTest.addIngredient(Measurement.GRAM, "Banana", tt);
        underTest.addIngredient(Measurement.PIECE, "Banana", tt);

        Assertions.assertEquals(1, underTest.getAllIngredients().size());
    }

    @Test
    public void shouldRemoveExistingIngredient() {
        //given
        Ingredient i1 = ValidObjectsPool.getValidIngredientOne();
        underTest.addIngredient(i1);

        //when
        boolean shouldBeTrue = underTest.deleteIngredient(i1.getName());

        //expect
        Assertions.assertTrue(shouldBeTrue);
    }

    @Test
    public void shouldThrowWhenRemovingWrongIngredient() {
        Ingredient i1 = ValidObjectsPool.getValidIngredientOne();
        Assertions.assertThrows(InvalidIngredientException.class, () -> underTest.deleteIngredient(i1.getName()));
    }
}
